/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	definition of the BackLog object, which can be used for direct jumps,
	and is also used for going to the previous file

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 03 December 1995
*/

#include "str_h"

#include "BackLog_h"
#include "HyperText_h"

#define MAX_BACKLOG 10

typedef struct {
	char filename[IO_MAXNAME];
	char directory[IO_MAXDIRECTORY];
	char title[TITLEBUFFER];
} Element;

struct _BackLog {
	int first, last;		/* the BackLog is actually built as a queue */
							/* first indicates the first element in the */
							/* BackLog, last indicates the next free element */
							/* the BackLog is empty when first==last */
							/* therefore, one element of the BackLog is */
							/* always unused */
	Element queue[MAX_BACKLOG];
};

Error BackLogInit(Global *g)
{
	return MEMAllocate(sizeof(BackLog),&g->backlog);
}

Error BackLogAdd(Global *g)
{
	BackLog *backlog=g->backlog;
	int pos=backlog->last;
	
	STRCopy(g->filename   ,backlog->queue[pos].filename);
	STRCopy(g->directory  ,backlog->queue[pos].directory);
	STRCopy(g->text->title,backlog->queue[pos].title);
	pos++;
	if (pos==MAX_BACKLOG) pos=0;
	backlog->last=pos;
	if (backlog->first==pos)
	{
		if ((pos+1)==MAX_BACKLOG)
			backlog->first=0;
		else
			backlog->first=pos+1;
	}
	return ERR_OK;
}

Error BackLogGet(Global *g)
{
	BackLog *backlog=g->backlog;
	int pos=backlog->last;

	if (pos==backlog->first) return ERR_ITNF;

	if (pos==0) pos=MAX_BACKLOG-1; else pos--;	/* the actual last item */
	backlog->last=pos;							/* which is the current file */
	if (pos==backlog->first) return ERR_ITNF;
	if (pos==0) pos=MAX_BACKLOG-1; else pos--;

	STRCopy(backlog->queue[pos].filename ,g->filename);
	STRCopy(backlog->queue[pos].directory,g->directory);
	
	return ERR_OK;
}

Error BackLogMenu(Global *g, PWObject menu)
{
	Error err;
	BackLog *backlog=g->backlog;
	int pos=backlog->first;

	catch( PWChange(menu,PW_OBJECT_AUXILIARY,g,NULL) );
	while (pos!=backlog->last)
	{
		catch( PWChange(menu,PW_MENU_ADD,backlog->queue[pos].title,NULL) );
		if (pos==MAX_BACKLOG) pos=0; else pos++;
	}
	return err;
}

Error BackLogSelect(PWObject object, char *item)
{
	Global *g;
	Element *element=(Element *)(item-offsetof(Element,title));

	PWQuery(object,PW_OBJECT_AUXILIARY,&g);
	STRCopy(element->directory,g->directory);
	STRCopy(element->filename ,g->filename );
	g->loaded=TRUE;
	PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);

	return ERR_OK;
}
