#ifndef _READER_HTCHUNK_H
#define _READER_HTCHUNK_
/*
		CHUNK HANDLING: FLEXIBLE ARRAYS
		(c) COPYRIGHT MIT 1995.
		Please first read the full copyright statement in the file COPYRIGH.

	This module implements a flexible array. It is a general utility module.  A
	chunk is a structure which may be extended. These routines create and
	append data to chunks, automatically reallocating them as necessary. It is
	garanteed that the array is '\0' terminated at all times, so the
	terminating function, HTChunkTerminate is only necessary to adjust the size
	in the chunk structure (the '\0' counts as a character when counting the
	size of the chunk.
   
	This module is a part of the W3C Reference Library.
   
	adjusted for use in ProWesS reader
	by Joachim Van der Auwera
	for PROGS, Professional & Graphical Software
	started 15 November 1995
*/

typedef struct {
	Size	size;						/* in bytes */
	Size	growby; 					/* allocation unit in bytes */
	Size	allocated;					/* current size of *data */
	char *	data;						/* allocated area or 0 */
} HTChunk;


/* Create new chunk 														*/
/* ON ENTRY 																*/
/* growby	The number of bytes to allocate at a time when the chunk is 	*/
/*			later extended. Arbitrary but normally a trade-off time vs. 	*/
/*			memory															*/
/* ON EXIT																	*/
/* returns	A chunk pointer to the new chunk								*/

extern Error HTChunkCreate(Size growby, HTChunk **ret);

/* Free a chunk 															*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* ON EXIT, 																*/
/* ch		is invalid and may not be used. 								*/
/* Error==ERR_OK															*/

extern Error HTChunkFree(HTChunk *ch);

/* Clear a chunk															*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* ON EXIT, 																*/
/* *ch		The size of the chunk is zero.									*/
/* Error==ERR_OK															*/

extern Error HTChunkClear(HTChunk *ch);

/* Ensure a chunk has a certain space in									*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* s		The size required												*/
/* ON EXIT, 																*/
/* *ch		Has size at least s 											*/

extern Error HTChunkEnsure(HTChunk *ch, Size needed);

/* Append a character to a	chunk											*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* c		The character to be appended									*/
/* ON EXIT, 																*/
/* *ch		Is one character bigger 										*/

extern Error HTChunkPutc(HTChunk *ch, char c);

/* Append a string to a  chunk												*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* str		points to a zero-terminated string to be appended				*/
/* ON EXIT, 																*/
/* *ch		Is bigger by strlen(str)										*/
						 
extern Error HTChunkPuts(HTChunk *ch, const char *str);

/* Append a zero character to a  chunk										*/
/* ON ENTRY,																*/
/* ch		A valid chunk pointer made by HTChunkCreate()					*/
/* ON EXIT, 																*/
/* *ch		Is one character bigger 										*/

extern Error HTChunkTerminate(HTChunk *ch);

#endif _READER_HTCHUNK_H
