#ifndef _READER_READER_H
#define _READER_READER_H
/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 17 November 1995
*/

#include <io_h>
#include <job_h>
#include <ProWesS_h>
#include "style_h"

typedef struct _HyperText HyperText;
typedef struct _TextLine TextLine;
typedef struct _BookMarks BookMarks;

typedef struct _BackLog BackLog;

#define catch(x) if (err=(x)) return err; else

#define MAX_MESSAGE 60

typedef struct {
	PWObject outline;				/* outline of the reader main window */
	PWObject applic;				/* application window */
	PWObject fsel;					/* file select object */
	PWObject title; 				/* title object */
	BackLog *backlog;
	BookMarks *bookmarks;
	Style *look;
	HyperText *text;
	char filename[IO_MAXFULLNAME];	/* name of current file (if any) */
	char directory[IO_MAXDIRECTORY];/* name of current directory (if any) */
	char position[IO_MAXFULLNAME];	/* position in file */
	pt canvaswidth; 				/* width of canvas to display file */
	char loaded;					/* TRUE when FILESELECT selected a file */
	pt Xpt2pix, Ypt2pix;			/* conversion factors */

	/* the applic window needs some data to figure out the pointer pos */
	char inlink;					/* TRUE when pointing at a link */
	pt ypos;						/* previous pointer position */
	pt yinline; 					/* y position of pointer in line */
	TextLine *line; 				/* line in which the pointer is */

	/* parameters for printing */
	char printing;					/* TRUE while printing */
	char printsub;					/* TRUE when subdocs should be included */
	char printmarg; 				/* TRUE when there should be a left margin */
	char drivername[PF_MAXDRIVERNAME];
	char devicename[IO_MAXFULLNAME];
	PWObject device,driver;
	int passes; 					/* no of passes for printing */
	Bbox pagebbox;					/* PageBboxFull for page */
	int pagenumber;
	int printfirst, printlast;		/* first and last page to be printed */

	/* parameters for bookmarks */
	PWObject menu;

	/* parameters for progress report windows */
	Job progres;					/* job for progress report */
	Job reader; 					/* job with ProWesS reader itself */
	char filetext[MAX_MESSAGE];
	char parttext[MAX_MESSAGE];
	char passtext[MAX_MESSAGE];
	char *actitext;
	PWObject file, part, pass, acti;
	Size filelength;
} Global;

Error ApplicCreate(Global *g);
Error ApplicRedisplay(Global *g);
Error ApplicPosition(Global *g, char *position);
Error FileDisplay(Global *g, char *position);

#endif _READER_READER_H
